#include "axn_plugin.h"
#include "cstrike.h"
#include "usrmsg.h"

// from cbase.h
#define DMG_NERVEGAS		(1 << 16)	// nerve toxins, very bad

#define MIN_HEALTH_THRESHOLD 1.0
	// minimum health threshhold for axn_gasimpactthreshold cvar
#define MIN_PANT_VOICE_INTERVAL 1.0
	// minimum possible interval (sec) between each pant voice (per player)
#define SMOKE_PUFF_LIFE 25.0
	// life (in sec) of smoke puff

#if defined __linux__
	#define EXTRAOFFSET		5 // offsets 5 higher in Linux builds
#else
	#define EXTRAOFFSET		0 // no change in Windows builds
#endif // defined __linux__

#define OFFSET_WEAPONTYPE	57 + EXTRAOFFSET


const char
	g_szSmokeGrenadeModel[] = "models/w_smokegrenade.mdl",
	g_szSmokeGrenadeClassname[] = "weapon_smokegrenade", // not used yet
	g_szGrenadeClassname[] = "grenade";
CSmokePuffList
	*g_pSmokePuffList = NULL;
//-----------------------------------------------------------------------------
CSmokePuff::CSmokePuff()
{
	m_iEntId = 0;
	m_vecOrigin.x = m_vecOrigin.y = m_vecOrigin.z = 0.0;
	UpdateTTD();
}
CSmokePuff::CSmokePuff(const Vector &vec, int index)
{
	m_iEntId = index;
	m_vecOrigin = vec;
	UpdateTTD();
}
CSmokePuff::CSmokePuff(const Vector &vec, edict_t *edict)
{
	m_iEntId = ENTINDEX(edict);
	m_vecOrigin = vec;
	UpdateTTD();
}
void CSmokePuff::UpdateTTD(void)
{
	m_fTTD = SetTimer(SMOKE_PUFF_LIFE);
}
bool CSmokePuff::KillMe(void)
{
	return gpGlobals->time >= m_fTTD;
}
//-----------------------------------------------------------------------------
// can be applied to "weapon_xxx" entities only
bool isSmokeGrenade(edict_t *pEdict) // not used here
{
	if(pEdict->pvPrivateData)
		return CSW_SMOKEGRENADE == *((int *)pEdict->pvPrivateData + OFFSET_WEAPONTYPE);
	return false;
}
//-----------------------------------------------------------------------------
bool isSmokeGrenadeExploded(const edict_t *pEdict)
{
	static int i, iEnt;

	iEnt = ENTINDEX(pEdict);
	if(iEnt > 0)
		for(i=g_pSmokePuffList->Count()-1; i>=0; i--)
		{
			if(g_pSmokePuffList->Get(i)->Data.m_iEntId == iEnt)
				return true;
		}

	return false;
}
//-----------------------------------------------------------------------------
bool isGrenade(edict_t *pEnt)
{
	return strcmp(STRING(pEnt->v.classname), g_szGrenadeClassname) == 0;
}
//-----------------------------------------------------------------------------
int CheckForSmokePuff(edict_t *pEdict, float sqr_radius)
{
	static int i, iCount;
	static float fVRadius;
	static CSmokePuff *pSmoke;

	fVRadius = axn_smokevradius->value;

	iCount = 0;
	for(i=g_pSmokePuffList->Count()-1; i>=0; i--)
	{
		pSmoke = &g_pSmokePuffList->Get(i)->Data;
		if(pSmoke->KillMe())
		{   // destroy smoke marker
			g_pSmokePuffList->Delete(i);
		}
		else 
		{
			// TODO: check for head position instead of origin?
			if(fabsf(pSmoke->m_vecOrigin.z - pEdict->v.origin.z) <= fVRadius && (pSmoke->m_vecOrigin - pEdict->v.origin).LengthSqr() <= sqr_radius)
				iCount++;
		}
	}
	
	return iCount;
}
//-----------------------------------------------------------------------------
// check for smoke grenade existence within the current raduis
int CheckForSmokeGrenade(edict_t *pEdict, float radius)
{
	static int iCount;
	static edict_t *pent;
	static Vector vecPlayerHead;
	
	iCount = 0;
	pent = NULL;
	vecPlayerHead = pEdict->v.origin + pEdict->v.view_ofs;
	
	while((pent=UTIL_FindEntityInSphere(pent, pEdict->v.origin, radius)) != NULL)
	{
		// NOTE: isSmokeGrenade() can not be applied to "grenade" entity :(
		if(isGrenade(pent) && (strcmpi(STRING(pent->v.model), g_szSmokeGrenadeModel) == 0))
		{
			// skip moving/non-exploded grenade or when player's head is above 
			// a grenade
			if(pent->v.velocity.LengthSqr() == 0.0 
			&& vecPlayerHead.z > pent->v.origin.z
			&& pent->v.waterlevel < 3
			&& isSmokeGrenadeExploded(pent))
			{
				if((pent->v.origin - pEdict->v.origin).Length() <= radius)
					iCount++;
			}
		}
	} // end while loop
	
	return iCount;
}
//-----------------------------------------------------------------------------
void CS_PlayerThink(edict_t *pEntity)
{
	static int index;
	
	index = ENTINDEX(pEntity); // ent id => player id

	if(g_PlayerInfo[index].player_wait <= gpGlobals->time)
	{
		if(IsAlive(pEntity) && pEntity->v.takedamage != DAMAGE_NO)
		{
			static float fSmokeRadius;
			static float fGasNadeRadius;

			fSmokeRadius = axn_smokeradius->value;
			fGasNadeRadius = axn_gasnaderadius->value;

			if(fSmokeRadius > 0.0 || fGasNadeRadius > 0.0)
			{
				static int iNadeCount; // actually it's impact quantity
				
				if(fSmokeRadius > 0.0)
				{
					fSmokeRadius *= fSmokeRadius;
					iNadeCount = CheckForSmokePuff(pEntity, fSmokeRadius);
				}
				else
					iNadeCount = 0;
				
				if(fGasNadeRadius > 0.0)
					iNadeCount += CheckForSmokeGrenade(pEntity, fGasNadeRadius);

				if(iNadeCount)
				{ 	// take health...
					static float fHealthTh;

					fHealthTh = axn_gasimpacttreshold->value;
					if(fHealthTh < MIN_HEALTH_THRESHOLD)
						fHealthTh = MIN_HEALTH_THRESHOLD;
						
					if(pEntity->v.health > fHealthTh
					&& axn_gasdamage->value > 0.0)
					{
						// get max possible damage for current cycle
						int iMaxDmg = pEntity->v.health - fHealthTh;
						int dmg = axn_gasdamage->value * iNadeCount;
						if(dmg > iMaxDmg)
							dmg = iMaxDmg;
						
						/*
						// try to send Damage message with DLL (see player.cpp)
						if(!pEntity->v.dmg_inflictor)
						{
							pEntity->v.dmg_take = dmg;
							pEntity->v.dmg_save = 0.0;
						}
						*/
						pEntity->v.health -= dmg;
						
						MESSAGE_BEGIN(MSG_ONE, gmsgDamage, NULL, &pEntity->v);
							WRITE_BYTE( 0.0 ); // dmg_save
							WRITE_BYTE( dmg ); // dmg_take
							WRITE_LONG( DMG_NERVEGAS ); // BUG: HUD shows RADIATION icon under CS or nothing at all
							WRITE_COORD( pEntity->v.origin.x ); // show dmg arrows all around
							WRITE_COORD( pEntity->v.origin.y );
							WRITE_COORD( pEntity->v.origin.z );
						MESSAGE_END();
						
						g_PlayerInfo[index].smkdmg += dmg;
					}
					
					if(axn_smokegasp->value > 0 
					&& (gpGlobals->time - g_PlayerInfo[index].last_pvoice_time) >= MIN_PANT_VOICE_INTERVAL)
					{	// play pant voice...
						switch(RANDOM_LONG(0, 1))
						{
						case 0: EMIT_SOUND_DYN2(pEntity, CHAN_VOICE, "player/gasp1.wav", RANDOM_FLOAT(0.3,0.8), RANDOM_FLOAT(1.0,1.5), 0, RANDOM_LONG(100,110)); break;
						case 1: EMIT_SOUND_DYN2(pEntity, CHAN_VOICE, "player/gasp2.wav", RANDOM_FLOAT(0.3,0.8), RANDOM_FLOAT(1.0,1.5), 0, RANDOM_LONG(100,110)); break;
						}

						g_PlayerInfo[index].last_pvoice_time = gpGlobals->time;
					}
				}
				else if(g_PlayerInfo[index].smkdmg > 0.0)
				{   // restore health...
					if(axn_gasrestoredamage->value != 0.0)
					{
						int dmg = g_PlayerInfo[index].smkdmg;

						// NOTE: if axn_gasrestoredamage < 0 then restore health at once
						if(axn_gasrestoredamage->value > 0.0) 
						{
							if(dmg > axn_gasrestoredamage->value)
								dmg = axn_gasrestoredamage->value;
					    }

						g_PlayerInfo[index].smkdmg -= dmg;
						pEntity->v.health += dmg;

						if(pEntity->v.health > pEntity->v.max_health) 
						{
							pEntity->v.health = pEntity->v.max_health;
							g_PlayerInfo[index].smkdmg = 0.0;
						}
						else if(g_PlayerInfo[index].smkdmg < 0.0)
							g_PlayerInfo[index].smkdmg = 0.0;
					}
					else
					{   // does not restore health at all
						g_PlayerInfo[index].smkdmg = 0.0;
					}
				}
			}
		}
		else
		{
			g_PlayerInfo[index].smkdmg = 0.0;
		}

		g_PlayerInfo[index].player_wait = SetTimer(axn_gasimpactcycle->value);
	}
}
//-----------------------------------------------------------------------------
