// vi: set ts=4 sw=4 :
// vim: set tw=75 :
// engine_api.cpp - implementation of Half-Life engine functions
/*
* Copyright(c)2001 Will Day <willday@hpgx.net>
*
* This file is part of Metamod.
*
* Metamod is free software; you can redistribute it and/or modify it
* under the terms of the GNU General Public License as published by the
* Free Software Foundation; either version 2 of the License, or(at
* your option)any later version.
*
* Metamod is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with Metamod; if not, write to the Free Software Foundation,
* Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* In addition, as a special exception, the author gives permission to
* link the code of this program with the Half-Life Game Engine("HL
* Engine")and Modified Game Libraries("MODs")developed by Valve,
* L.L.C("Valve"). You must obey the GNU General Public License in all
* respects for all of the code used other than the HL Engine and MODs
* from Valve. If you modify this file, you may extend this exception
* to your version of the file, but you are not obligated to do so. If
* you do not wish to do so, delete this exception statement from your
* version.
*
*/
#include "axn_plugin.h"
#include "cstrike.h"
#include "usrmsg.h"

#pragma warning(disable:4702) // unreachable code
#pragma warning(disable:4127) // conditional expression is constant


const char 
	g_sGasNadeSound[] = "weapons/sg_explode.wav",
	g_sRoundStart[] = "\"Round_Start\"";

enginefuncs_t 
	meta_engfuncs,
	meta_engfuncs_post;


edict_t* FindEntityByString(edict_t *pEdictStartSearchAfter, const char *pszField, const char *pszValue)
{
    bSpawn = false; // what for?

    RETURN_META_VALUE(MRES_IGNORED, (edict_t*)META_RESULT_PREVIOUS);
}

void AlertMessage(ALERT_TYPE atype, char *szFmt, ...)
{	// called under CS only
	if(atype == at_logged)
	{	// detect round start under CS...
		static int i;
		static char szBuffer[256];		

		va_list	varList;
		va_start(varList, szFmt);
		
		i = vsnprintf(szBuffer, 255, szFmt, varList);
		va_end(varList);
		if(i < 0)
			szBuffer[0] = 0;
		else
		{	
			szBuffer[i] = 0;
			if(strstr(szBuffer, g_sRoundStart))
			{				
				//LOG_CONSOLE(PLID, "Cleanup smokegrenades registration info: %d", g_pSmokePuffList->Count());
				g_pSmokePuffList->Clear();
			}
		}
	}

	RETURN_META(MRES_IGNORED);
}

void EmitSound(edict_t *entity, int channel, const char *sample, float volume, float attenuation, int fFlags, int pitch)
{	// called under CS only
	static SSmokePuffItem *pItem;

	if(isGrenade(entity) && strcmpi(sample, g_sGasNadeSound) == 0) 
	{   // register new smoke puff...
		pItem = g_pSmokePuffList->Add();
		if(pItem) {
			pItem->Data.m_vecOrigin = entity->v.origin;
			pItem->Data.m_iEntId = ENTINDEX(entity);
			//LOG_MESSAGE(PLID, "Registered new smoke at {%.3f, %.3f, %.3f}", pItem->Data.m_vecOrigin.x, pItem->Data.m_vecOrigin.y, pItem->Data.m_vecOrigin.z);
		}
	}

	RETURN_META(MRES_IGNORED);
}

void TraceLine(const float *v1, const float *v2, int fNoMonsters, edict_t *pentToSkip, TraceResult *ptr)
{
    TRACE_LINE(v1, v2, fNoMonsters, pentToSkip, ptr);
    
    if(ptr->iHitgroup == HITGROUP_HEAD && axn_noheadshot->value != 0)
    {   // no headshot
        ptr->iHitgroup = HITGROUP_GENERIC;
    }
    
    RETURN_META(MRES_SUPERCEDE);
}

void MessageEnd_Post(void)
{
    g_msg_type = MSGEND;
    g_msg_index = MSGEND;
    
    MsgFunc(0);
    MsgFunc = MsgFunc_Void; // reset
    
    RETURN_META(MRES_IGNORED);
}

void MessageBegin(int msg_dest, int msg_type, const float *pOrigin, edict_t *ed)
{
    // ALERT(at_console,"MessageBegin %s\n",GET_USER_MSG_NAME(&Plugin_info,msg_type,0));
    g_msg_type = msg_type;
    g_msg_index = MSGBEGIN;
    MsgFunc = MsgFunc_Void;

    if(msg_type==gmsgCurWeapon){MsgFunc=MsgFunc_CurWeapon;}
    else if(msg_type==gmsgDamage){MsgFunc=MsgFunc_Damage;}
    else if(msg_type==gmsgStatusText){MsgFunc=MsgFunc_StatusText;}
    else if(msg_type==gmsgScreenFade){MsgFunc=MsgFunc_ScreenFade;}
    else if(msg_type==gmsgWeaponList){MsgFunc=MsgFunc_WeaponList;}
    //else if(msg_type==gmsgTextMsg){MsgFunc=MsgFunc_TextMsg;}
    
    if(MsgFunc((void*)ed))
    {
        RETURN_META(MRES_SUPERCEDE);
    }
    else
    {
        RETURN_META(MRES_IGNORED);
    }
}

void WriteByte(int iValue)
{
    g_msg_index++;
    if(MsgFunc((void*)&iValue))
    {
        WRITE_BYTE(iValue);
        RETURN_META(MRES_SUPERCEDE);
    }
    else
    {
        RETURN_META(MRES_IGNORED);
    }
}

void WriteChar(int iValue)
{
    g_msg_index++;
    if(MsgFunc((void*)&iValue))
    {
        WRITE_CHAR(iValue);
        RETURN_META(MRES_SUPERCEDE);
    }
    else
    {
        RETURN_META(MRES_IGNORED);
    }
}

void WriteShort(int iValue)
{
    g_msg_index++;
    if(MsgFunc((void*)&iValue))
    {
        WRITE_SHORT(iValue);
        RETURN_META(MRES_SUPERCEDE);
    }
    else
    {
        RETURN_META(MRES_IGNORED);
    }
}

void WriteLong(int iValue)
{
    g_msg_index++;
    if(MsgFunc((void*)&iValue))
    {
        WRITE_LONG(iValue);
        RETURN_META(MRES_SUPERCEDE);
    }
    else
    {
        RETURN_META(MRES_IGNORED);
    }
}

void WriteAngle(float flValue)
{
    g_msg_index++;
    if(MsgFunc((void*)&flValue))
    {
        WRITE_ANGLE(flValue);
        RETURN_META(MRES_SUPERCEDE);
    }
    else
    {
        RETURN_META(MRES_IGNORED);
    }
}

void WriteCoord(float flValue)
{
    g_msg_index++;
    if(MsgFunc((void*)&flValue))
    {
        WRITE_COORD(flValue);
        RETURN_META(MRES_SUPERCEDE);
    }
    else
    {
        RETURN_META(MRES_IGNORED);
    }
}

void WriteString(const char *sz)
{
    static char string[1025];

    g_msg_index++;
    strncpy(string, sz, 1024);
	if(string[1024]) // for safety
		string[1024] = '\0';
    
    if(MsgFunc((void*)string))
    {
        WRITE_STRING(string);
        RETURN_META(MRES_SUPERCEDE);
    }
    else
    {
        RETURN_META(MRES_IGNORED);
    }
}

void WriteEntity(int iValue)
{
    g_msg_index++;
    if(MsgFunc((void*)&iValue))
    {
        WRITE_ENTITY(iValue);
        RETURN_META(MRES_SUPERCEDE);
    }
    else
    {
        RETURN_META(MRES_IGNORED);
    }
}

int RegUserMsg_Post(const char *pszName, int iSize)
{
    int msg_type = META_RESULT_ORIG_RET(int);
    
    // ALERT(at_console,"pfnRegUserMsg %s %d\n",pszName,msg_type);
    
    // initialize global msg IDs...
    if(strcmp(pszName,"HudText")==0) gmsgHudText = msg_type;
    else if(strcmp(pszName,"SayText")==0) gmsgSayText = msg_type;
    else if(strcmp(pszName,"Damage")==0) gmsgDamage=msg_type;
    else if(strcmp(pszName,"StatusText")==0) gmsgStatusText=msg_type;
    else if(strcmp(pszName,"ScreenShake")==0) gmsgScreenShake=msg_type;
    else if(strcmp(pszName,"ScreenFade")==0) gmsgScreenFade=msg_type;
    else if(strcmp(pszName,"CurWeapon")==0) gmsgCurWeapon=msg_type;
    else if(strcmp(pszName,"WeaponList")==0) gmsgWeaponList=msg_type;
    else if(strcmp(pszName,"TextMsg")==0) gmsgTextMsg=msg_type;

    RETURN_META_VALUE(MRES_IGNORED, META_RESULT_PREVIOUS);
}

void SetPhysicsKeyValue( const edict_t *pClient, const char *key, const char *value )
{
    //  ALERT(at_console,"SetPhysicsKeyValue %d %s %s\n",ENTINDEX((edict_t *)pClient),key,value);
    if(strcmp(key, "hl") == 0)
    {	// on player spawn trigger
        Restore((edict_t *)pClient);
    }
    RETURN_META(MRES_IGNORED);
}

C_DLLEXPORT int GetEngineFunctions(enginefuncs_t *pengfuncsFromEngine, int *interfaceVersion)
{
    LOG_DEVELOPER(PLID, "called: GetEngineFunctions; version=%d", *interfaceVersion);
    if(!pengfuncsFromEngine){
        LOG_ERROR(PLID, "GetEngineFunctions called with null pengfuncsFromEngine");
        return(FALSE);
    }
    else if(*interfaceVersion!=ENGINE_INTERFACE_VERSION){
        LOG_ERROR(PLID, "GetEngineFunctions version mismatch; requested=%d ours=%d", *interfaceVersion, ENGINE_INTERFACE_VERSION);
        // Tell metamod what version we had, so it can figure out who is out of date.
        *interfaceVersion=ENGINE_INTERFACE_VERSION;
        return(FALSE);
    }

    // register engine forward functions...
    meta_engfuncs.pfnFindEntityByString = FindEntityByString;
    meta_engfuncs.pfnTraceLine = TraceLine;
    meta_engfuncs.pfnMessageBegin = MessageBegin;
    
    if(g_mod_type == CSTRIKE)
    {
    	meta_engfuncs.pfnEmitSound = EmitSound;
    	meta_engfuncs.pfnAlertMessage = AlertMessage;
	}
    
    meta_engfuncs.pfnWriteByte=WriteByte;
    meta_engfuncs.pfnWriteChar=WriteChar;
    meta_engfuncs.pfnWriteShort=WriteShort;
    meta_engfuncs.pfnWriteLong=WriteLong;
    meta_engfuncs.pfnWriteAngle=WriteAngle;
    meta_engfuncs.pfnWriteCoord=WriteCoord;
    meta_engfuncs.pfnWriteString=WriteString;
    meta_engfuncs.pfnWriteEntity=WriteEntity;
    meta_engfuncs.pfnSetPhysicsKeyValue=SetPhysicsKeyValue;
    
    memcpy(pengfuncsFromEngine, &meta_engfuncs, sizeof(enginefuncs_t));
    
    return TRUE;
}

C_DLLEXPORT int GetEngineFunctions_Post(enginefuncs_t *pengfuncsFromEngine, int *interfaceVersion)
{
    LOG_DEVELOPER(PLID, "called: GetEngineFunctions; version=%d", *interfaceVersion);
    if(!pengfuncsFromEngine){
        LOG_ERROR(PLID, "GetEngineFunctions called with null pengfuncsFromEngine");
        return(FALSE);
    }
    else if(*interfaceVersion!=ENGINE_INTERFACE_VERSION){
        LOG_ERROR(PLID, "GetEngineFunctions version mismatch; requested=%d ours=%d", *interfaceVersion, ENGINE_INTERFACE_VERSION);
        // Tell metamod what version we had, so it can figure out who is out of date.
        *interfaceVersion=ENGINE_INTERFACE_VERSION;
        return(FALSE);
    }
    
    meta_engfuncs_post.pfnRegUserMsg = RegUserMsg_Post;
    meta_engfuncs_post.pfnMessageEnd = MessageEnd_Post;
    
    memcpy(pengfuncsFromEngine, &meta_engfuncs_post, sizeof(enginefuncs_t));
    
    return TRUE;
}
