#ifndef ListT
#define ListT

#include "list.h"
#include "list_rc.h"
//---------------------------------------------------------------------------
template <class T> CDataList<T>::CDataList()
{
  m_pHead = NULL;
  m_pTail = NULL;

  m_iCount = 0;
}
//---------------------------------------------------------------------------
template <class T> SListItem<T>* CDataList<T>::NewItem(void)
{
  SListItem<T> *pNewItem = new SListItem<T>;

  if(pNewItem)
    pNewItem->Next = NULL;

  return pNewItem;
}
//---------------------------------------------------------------------------
template <class T> void CDataList<T>::AppendItem(SListItem<T> *Item)
{
  if(m_iCount)
    m_pTail->Next = Item;
  else
    m_pHead = Item;

  m_pTail = Item;
  m_iCount++;
}
//---------------------------------------------------------------------------
template <class T> void CDataList<T>::GetPare(int Index, SListItem<T> **pPrev, SListItem<T> **pItem)
{
  *pPrev = NULL;
  *pItem = m_pHead;

  for(int i=0; i<Index; i++)
  {
    *pPrev = *pItem;
    *pItem = (*pItem)->Next;
  }
}
//---------------------------------------------------------------------------
template <class T> void CDataList<T>::InsertItem(int Index, SListItem<T> *Item)
{
  SListItem<T> *pItem, *pPrev;

  if(Index < 0 || Index > m_iCount)
  {
    int iMax = m_iCount;
    if(iMax)
      iMax--;
    CDataList<T>::Error(g_szListBadIndex, "CDataList<T>::InsertItem()", Index, iMax);
    return;
  }

  GetPare(Index, &pPrev, &pItem);

  if(pPrev)
  {
    if(!pPrev->Next)
      m_pTail = Item;
    Item->Next = pPrev->Next;
    pPrev->Next = Item;
  }
  else
  {
    Item->Next = m_pHead;
    m_pHead = Item;
    if(!m_iCount)
      m_pTail = Item;
  }

  m_iCount++;
}
//---------------------------------------------------------------------------
template <class T> void CDataList<T>::DeleteItem(SListItem<T> *pPrev, SListItem<T> *pItem)
{
  m_iCount--;

  if(pPrev)
  { 
    pPrev->Next = pItem->Next;
    if(!pItem->Next)
      m_pTail = pPrev;
  }
  else
  { 
    m_pHead = pItem->Next;
    if(!m_iCount)
      m_pTail = NULL;
  }

  delete pItem;
}
//---------------------------------------------------------------------------
template <class T> bool CDataList<T>::Add(T &Data)
{
  SListItem<T> *pNewItem = NewItem();

  if(pNewItem)
  {
    // op '=' should be overloaded for structure!
    pNewItem->Data = Data;
    AppendItem(pNewItem);

    return true;
  }

  return false;
}
//---------------------------------------------------------------------------
template <class T> T* CDataList<T>::Add(void)
{
  SListItem<T> *pNewItem = NewItem();

  if(pNewItem)
  {
    AppendItem(pNewItem);
    return &pNewItem->Data;
  }

  return NULL;
}
//---------------------------------------------------------------------------
template <class T> SListItem<T>* CDataList<T>::GetItem(int Index)
{
  SListItem<T> *pResult = NULL;

  if(!m_iCount) {
    CDataList<T>::Error(g_szListEmpty, "CDataList<T>::GetItem()");
    return NULL;
  }
  if(Index < 0 || Index >= m_iCount) {
    CDataList<T>::Error(g_szListBadIndex, "CDataList<T>::GetItem()", Index, m_iCount-1);
    return NULL;
  }

  if(Index == m_iCount - 1)
    pResult = m_pTail;
  else
  {
    pResult = m_pHead;

    for(int i=0; i<Index; i++)
      pResult = pResult->Next;
  }

  return pResult;
}
//---------------------------------------------------------------------------
template <class T> T* CDataList<T>::Get(int Index)
{
  SListItem<T> *pItem = GetItem(Index);

  if(pItem)
    return &pItem->Data;
  return NULL;
}
//---------------------------------------------------------------------------
template <class T> T CDataList<T>::Extract(int Index)
{
  SListItem<T> *pItem = GetItem(Index);

  if(pItem)
    return pItem->Data;
  
  CDataList<T>::Error(g_szListBadItem, "CDataList<T>::Extract()", Index);

  return NULL;
}
//---------------------------------------------------------------------------
template <class T> T* CDataList<T>::Insert(int Index)
{
  SListItem<T> *pNewItem = NewItem();

  if(pNewItem)
  {
    InsertItem(Index, pNewItem);
    return &pNewItem->Data;
  }

  return NULL;
}
//---------------------------------------------------------------------------
template <class T> bool CDataList<T>::Insert(int Index, T &Data)
{
  SListItem<T> *pNewItem = NewItem();

  if(pNewItem)
  {
    pNewItem->Data = Data;
    InsertItem(Index, pNewItem);

    return true;
  }

  return false;
}
//---------------------------------------------------------------------------
template <class T> void CDataList<T>::Delete(int Index)
{
  if(!m_iCount) {
    CDataList<T>::Error(g_szListEmpty, "CDataList<T>::Delete()");
    return;
  }
  if(Index < 0 || Index >= m_iCount) {
    CDataList<T>::Error(g_szListBadIndex, "CDataList<T>::Delete()", Index, m_iCount-1);
    return;
  }

  SListItem<T> *pItem, *pPrev;

  GetPare(Index, &pPrev, &pItem);
  DeleteItem(pPrev, pItem);
}
//---------------------------------------------------------------------------
template <class T> bool CDataList<T>::Remove(T &Data)
{
  SListItem<T> *pItem = m_pHead, *pPrev = NULL;

  for(int i=0; i<m_iCount; i++)
  {
    if(pItem->Data == Data)
    {
      DeleteItem(pPrev, pItem);
      return true;
    }

    pPrev = pItem;
    pItem = pItem->Next;
  }

  return false;
}
//---------------------------------------------------------------------------
template <class T> bool CDataList<T>::Remove(T* pData)
{
  SListItem<T> *pItem = m_pHead, *pPrev = NULL;

  for(int i=0; i<m_iCount; i++)
  {
    if(&pItem->Data == pData)
    {
      DeleteItem(pPrev, pItem);
      return true;
    }

    pPrev = pItem;
    pItem = pItem->Next;
  }

  return false;
}
//---------------------------------------------------------------------------
template <class T>  void CDataList<T>::Clear(void)
{
  for(int i=0; i<m_iCount; i++)
  {
    if(!m_pHead) {
      CDataList<T>::Error(g_szListError, "CDataList<T>::Clear():m_pHead == NULL");
      return;
	}

    m_pTail = m_pHead->Next;
    delete m_pHead;
    m_pHead = m_pTail;
  }

  m_iCount = 0;
  m_pHead = m_pTail = NULL;
}
//---------------------------------------------------------------------------
template <class T> int CDataList<T>::Count(void)
{
	return m_iCount;
}
//---------------------------------------------------------------------------
template <class T> void CDataList<T>::Error(
  const char *Msg,
  const char *Within,
  ...)
{
	char sErrMsg[128], sBuffer[256];
	va_list varList;

	va_start(varList, Within);
	vsprintf(sErrMsg, Msg, varList);
	va_end(varList);

	sprintf(sBuffer, "%s\nin %s", sErrMsg, Within);

	LOG_ERROR(PLID, sBuffer);
}
//---------------------------------------------------------------------------
#endif
