// vi: set ts=4 sw=4 :
// vim: set tw=75 :
// Selected portions of dlls/util.cpp from SDK 2.1.
// Functions copied from there as needed...
/***
*
* Copyright(c)1999, 2000 Valve LLC. All rights reserved.
*
* This product contains software technology licensed from Id
* Software, Inc.("Id Technology"). Id Technology(c)1996 Id Software, Inc.
* All Rights Reserved.
*
* Use, distribution, and modification of this source code and/or resulting
* object code is restricted to non-commercial enhancements to products from
* Valve LLC. All other use, distribution, or modification is prohibited
* without written permission from Valve LLC.
*
****/
/*
=====util.cpp========================================================
Utility code. Really not optional after all.
*/
#include <extdll.h>
#include <meta_api.h> // of course
#include <enginecallback.h> // ALERT()
#include "sdk_util.h"
#include "progdefs.h"
#include "edict.h"
#include "const.h"
#include "cbase.h"
#include "shake.h"
#include "weaponinfo.h"
#include "progdefs.h"
#include "scriplib.h"
#include "axn_plugin.h"
#include "entity.h"

#include "usrmsg.h"

//=========================================================
// UTIL_LogPrintf - Prints a logged message to console.
// Preceded by LOG:(timestamp)< message>
//=========================================================
void UTIL_LogPrintf(char *fmt, ...)
{
	va_list argptr;
	static char string[1024];
	va_start(argptr, fmt);
	vsprintf(string, fmt, argptr);
	va_end(argptr);
	// Print to server console
	ALERT(at_logged, "%s", string);
}

float UTIL_AngleDiff(float destAngle, float srcAngle)
{
	float delta;
	delta=destAngle - srcAngle;
	if(destAngle>srcAngle)
	{
		if(delta>=180)
			delta-=360;
	}
	else
	{
		if(delta<=-180)
			delta+=360;
	}
	return delta;
}

Vector UTIL_VecToAngles(const Vector &vec)
{
	float rgflVecOut[3];
	VEC_TO_ANGLES(vec, rgflVecOut);
	return Vector(rgflVecOut);
}

// Overloaded to add IGNORE_GLASS
void UTIL_TraceLine(const Vector &vecStart, const Vector &vecEnd, IGNORE_MONSTERS igmon, IGNORE_GLASS ignoreGlass, edict_t *pentIgnore, TraceResult *ptr)
{
	TRACE_LINE(vecStart, vecEnd,(igmon==ignore_monsters ? TRUE : FALSE)|(ignoreGlass?0x100:0), pentIgnore, ptr);
}

void UTIL_TraceLine(const Vector &vecStart, const Vector &vecEnd, IGNORE_MONSTERS igmon, edict_t *pentIgnore, TraceResult *ptr)
{
	TRACE_LINE(vecStart, vecEnd,(igmon==ignore_monsters ? TRUE : FALSE), pentIgnore, ptr);
}

void UTIL_MakeVectors(const Vector &vecAngles)
{
	MAKE_VECTORS(vecAngles);
}

edict_t *UTIL_FindEntityInSphere(edict_t *pentStart, const Vector &vecCenter, float flRadius)
{
	edict_t *pentEntity;
	pentEntity=FIND_ENTITY_IN_SPHERE(pentStart, vecCenter, flRadius);
	if(!FNullEnt(pentEntity))
		return pentEntity;
	return NULL;
}

edict_t *UTIL_FindEntityByString(edict_t *pentStart, const char *szKeyword, const char *szValue)
{
	edict_t *pentEntity;
	pentEntity=FIND_ENTITY_BY_STRING(pentStart, szKeyword, szValue);
	if(!FNullEnt(pentEntity))
		return pentEntity;
	return NULL;
}

edict_t *UTIL_FindEntityByClassname(edict_t *pentStart, const char *szName)
{
	return UTIL_FindEntityByString(pentStart, "classname", szName);
}

unsigned short FixedUnsigned16( float value, float scale )
{
	int output;

	output = value * scale;
	if ( output < 0 )
		output = 0;
	if ( output > 0xFFFF )
		output = 0xFFFF;

	return (unsigned short)output;
}

short FixedSigned16( float value, float scale )
{
	int output;

	output = value * scale;

	if ( output > 32767 )
		output = 32767;

	if ( output < -32768 )
		output = -32768;

	return (short)output;
}

void UTIL_HudMessage( edict_t *pEdict, const hudtextparms_t &textparms, const char *pMessage )
{
	if ( !UTIL_IsValidEntity( pEdict ) )
		return;

	MESSAGE_BEGIN( MSG_ONE, SVC_TEMPENTITY, NULL, pEdict );
		WRITE_BYTE( TE_TEXTMESSAGE );
		WRITE_BYTE( textparms.channel & 0xFF );

		WRITE_SHORT( FixedSigned16( textparms.x, 1<<13 ) );
		WRITE_SHORT( FixedSigned16( textparms.y, 1<<13 ) );
		WRITE_BYTE( textparms.effect );

		WRITE_BYTE( textparms.r1 );
		WRITE_BYTE( textparms.g1 );
		WRITE_BYTE( textparms.b1 );
		WRITE_BYTE( textparms.a1 );

		WRITE_BYTE( textparms.r2 );
		WRITE_BYTE( textparms.g2 );
		WRITE_BYTE( textparms.b2 );
		WRITE_BYTE( textparms.a2 );

		WRITE_SHORT( FixedUnsigned16( textparms.fadeinTime, 1<<8 ) );
		WRITE_SHORT( FixedUnsigned16( textparms.fadeoutTime, 1<<8 ) );
		WRITE_SHORT( FixedUnsigned16( textparms.holdTime, 1<<8 ) );

		if ( textparms.effect == 2 )
			WRITE_SHORT( FixedUnsigned16( textparms.fxTime, 1<<8 ) );
		
		if ( strlen( pMessage ) < 512 )
		{
			WRITE_STRING( pMessage );
		}
		else
		{
			char tmp[512];
			strncpy( tmp, pMessage, 511 );
			tmp[511] = 0;
			WRITE_STRING( tmp );
		}
	MESSAGE_END();
}

void UTIL_SayTextAll(const char *pText, edict_t *pEdict)
{
	if(!UTIL_IsValidEntity(pEdict))
		return;
	MESSAGE_BEGIN(MSG_ALL, gmsgSayText, NULL);
		WRITE_BYTE(ENTINDEX(pEdict));
		WRITE_STRING(pText);
	MESSAGE_END();
}

BOOL UTIL_IsValidEntity(edict_t *pent)
{
	if(!pent || pent->free ||(pent->v.flags & FL_KILLME)||!(pent->v.flags & FL_CLIENT))
		return FALSE;
	return TRUE;
}

BOOL IsAlive(edict_t *pEdict)
{
	return((pEdict->v.deadflag==DEAD_NO)&&
		(pEdict->v.health>0)&& !(pEdict->v.flags & FL_NOTARGET));
}

#include <stdarg.h>
#include <string.h>
char* UTIL_VarArgs(char *format, ...)
{
	va_list argptr;
	static char string[1024];
	va_start(argptr, format);
	vsprintf(string, format,argptr);
	va_end(argptr);
	return string;
}

void UTIL_StringToVector(float *pVector, const char *pString)
{
	char *pstr, *pfront, tempString[128];
	int j;
	strcpy(tempString, pString);
	pstr=pfront=tempString;
	for(j=0; j<3; j++)// lifted from pr_edict.c
	{
		pVector[j]=atof(pfront);
		while(*pstr && *pstr!=' ')
			pstr++;
		if(!*pstr)
			break;
		pstr++;
		pfront=pstr;
	}
	if(j<2)
	{
	/*
	ALERT(at_error, "Bad field in entity!! %s:%s==\"%s\"\n",
	pkvd->szClassName, pkvd->szKeyName, pkvd->szValue);
		*/
		for(j=j+1;j<3; j++)
			pVector[j]=0;
	}
}
