#include "axn_plugin.h"
#include "usrmsg.h"
#include "cstrike.h"


#define MAX_WEAPONS 32

// from amxmodx...
#if defined __linux__
	#define EXTRAOFFSET_WEAPONS	4 // weapon offsets are obviously only 4 steps higher on Linux!
#else
	#define EXTRAOFFSET_WEAPONS	0
#endif // defined __linux__

// from amxmodx...
#define OFFSET_WEAPONTYPE			43 + EXTRAOFFSET_WEAPONS
#define OFFSET_SILENCER_FIREMODE	74 + EXTRAOFFSET_WEAPONS
// from amxmodx...
#define M4A1_SILENCED					(1<<2)
#define USP_SILENCED					(1<<0)
// from amxmodx...
#define GLOCK_BURSTMODE	2
#define FAMAS_BURSTMODE	16

// angle (in degrees) of bullets spread...
#define SHOTGUN_SPREAD 3
#define BURST_SPREAD 1

#define VALVE_WEAPON_SHOTGUN  7
#define VALVE_WEAPON_CROSSBOW 6

const char g_szClassname[] = "classname";

int gmsgCurWeapon;
int gmsgDamage;
int gmsgStatusText;
int gmsgScreenShake;
int gmsgScreenFade;
int gmsgWeaponList;
int gmsgHudText;
int gmsgSayText;
int gmsgTextMsg;

// initialized by MsgFunc_WeaponList()
int Weapons[MAX_WEAPONS]; // [weapon_id] = slot_id
char g_szWeaponNames[MAX_WEAPONS][32];

float fTanTable[11] = {
	0.0, 
	0.01745506492821758577,
	0.03492076949174773050,
	0.05240777928304120404,
	0.06992681194351041367,
	0.08748866352592400522,  // 5 degrees
	0.10510423526567646251,
	0.12278456090290459113,
	0.14054083470239144683,
	0.15838444032453629384,
	0.17632698070846497347   // 10 degrees
};

pfnMsgFunc MsgFunc = NULL;
//-----------------------------------------------------------------------------
bool isSniperWeapon(int wid)
{
	switch(g_mod_type)
	{
	case HLMOD:
		return wid == VALVE_WEAPON_CROSSBOW;
	case CSTRIKE:
		return wid == CSW_AWP || wid == CSW_SCOUT || wid == CSW_SG550 || wid == CSW_G3SG1;
	}

	return false;
}
//-----------------------------------------------------------------------------
bool isWeaponSilenced(int owner, int wid)
{
	static edict_t *pEnt, *pOwnerEnt;
	static char *pszWeaponName;

	if(g_mod_type != CSTRIKE || (wid != CSW_M4A1 && wid != CSW_USP))
		return false;

	pEnt = NULL;
	pOwnerEnt = INDEXENT(owner);
	pszWeaponName = g_szWeaponNames[wid];

	do {
		pEnt = FIND_ENTITY_BY_STRING(pEnt, g_szClassname, pszWeaponName);
	} while(pEnt && (pEnt->v.owner != pOwnerEnt));

	if(pEnt)
	{
		static int *pintSilenceMode;
		
		pintSilenceMode = (int*)pEnt->pvPrivateData + OFFSET_SILENCER_FIREMODE;

		if(wid == CSW_M4A1)
			return *pintSilenceMode & M4A1_SILENCED;
		return *pintSilenceMode & USP_SILENCED;
	}

	return false;
}
//-----------------------------------------------------------------------------
bool isWeaponInBurstMode(int owner, int wid)
{
	static edict_t *pEnt, *pOwnerEnt;
	static char *pszWeaponName;

	if(g_mod_type != CSTRIKE || (wid != CSW_GLOCK18 && wid != CSW_FAMAS))
		return false;

	pEnt = NULL;
	pOwnerEnt = INDEXENT(owner);
	pszWeaponName = g_szWeaponNames[wid];

	do {
		pEnt = FIND_ENTITY_BY_STRING(pEnt, g_szClassname, pszWeaponName);
	} while(pEnt && (pEnt->v.owner != pOwnerEnt));

	if(pEnt)
	{
		static int *pintValue;
		
		pintValue = (int*)pEnt->pvPrivateData + OFFSET_SILENCER_FIREMODE;

		if(wid == CSW_FAMAS)
			return *pintValue == FAMAS_BURSTMODE;
		return *pintValue == GLOCK_BURSTMODE;
	}

	return false;
}
//-----------------------------------------------------------------------------
bool isShotgun(int wid)
{
	switch(g_mod_type)
	{
	case HLMOD:
		return wid == VALVE_WEAPON_SHOTGUN;
	case CSTRIKE:
		return wid == CSW_M3 || wid == CSW_XM1014;
	}
	
	return false;
}
//-----------------------------------------------------------------------------
void SendTracer(int msg_type, edict_t *ent, Vector &vecStart, Vector &vecEnd, bool &recalc)
{
	// NOTE: recalc not used here

	MESSAGE_BEGIN(msg_type, SVC_TEMPENTITY, vecStart, ent);
		WRITE_BYTE(TE_TRACER);
		WRITE_COORD(vecStart.x);
		WRITE_COORD(vecStart.y);
		WRITE_COORD(vecStart.z);
		WRITE_COORD(vecEnd.x);
		WRITE_COORD(vecEnd.y);
		WRITE_COORD(vecEnd.z);
	MESSAGE_END();
}
//-----------------------------------------------------------------------------
void SendTracerBurst(int msg_type, edict_t *ent, Vector &vecStart, Vector &vecEnd, bool &recalc)
{
	static float fSpread;
	static Vector vecTemp;

	SendTracer(msg_type, ent, vecStart, vecEnd, recalc);

	if(recalc)
	{
		fSpread = fTanTable[BURST_SPREAD] * (vecEnd - vecStart).Length();
		recalc = false;
	}
	
	// TODO: rewrite this to make more real and smooth
	vecTemp = vecEnd;
	SendTracer(msg_type, ent, vecStart, vecTemp, recalc);
	vecTemp[0] += fSpread;
	vecTemp[1] += fSpread;
	vecTemp[2] += fSpread;
	SendTracer(msg_type, ent, vecStart, vecTemp, recalc);
	vecTemp = vecEnd;
	vecEnd[0] -= fSpread;
	vecEnd[1] -= fSpread;
	vecEnd[2] -= fSpread;
	SendTracer(msg_type, ent, vecStart, vecTemp, recalc);
}
//-----------------------------------------------------------------------------
void SendTracerShortgun(int msg_type, edict_t *ent, Vector &vecStart, Vector &vecEnd, bool &recalc)
{
	static float fSpread;
	static Vector vecStore;
	
	SendTracer(msg_type, ent, vecStart, vecEnd, recalc);

	if(recalc)
	{
		fSpread = fTanTable[SHOTGUN_SPREAD] * (vecEnd - vecStart).Length();
		recalc = false;
	}

	// TODO: rewrite this to make more real and smooth
	vecStore = vecEnd;
	vecStore[2] += fSpread;
	SendTracer(msg_type, ent, vecStart, vecStore, recalc);
	vecStore[1] += fSpread;
	SendTracer(msg_type, ent, vecStart, vecStore, recalc);
	vecStore[0] += fSpread;
	SendTracer(msg_type, ent, vecStart, vecStore, recalc);
	vecStore = vecEnd;
	vecStore[0] -= fSpread;
	SendTracer(msg_type, ent, vecStart, vecStore, recalc);
	vecStore[1] -= fSpread;
	SendTracer(msg_type, ent, vecStart, vecStore, recalc);
	vecStore[2] -= fSpread;
	SendTracer(msg_type, ent, vecStart, vecStore, recalc);
}
//-----------------------------------------------------------------------------
bool MsgFunc_Void(void *value)
{
	return false;
}
//-----------------------------------------------------------------------------
bool MsgFunc_CurWeapon(void *value)
{
	static int 
		i, index = 0,
		id, // id - weapon id
		clip = 0,
		tracer, tracer_flags;
	static edict_t *pEntity;

	switch(g_msg_index)
	{
	case MSGBEGIN:
		pEntity = (edict_t*)value;
		index = ENTINDEX(pEntity);
		break;
	case 1: // 0 - hide weapon; 1 - select/fire weapon
		i = *(int*)value;
		break;
	case 2: // weapon id
		id=*((int*)value);
		break;
	case 3: // -1 - clip does not exists
		clip=*((int*)value);
		break;
	case MSGEND:
		tracer = axn_tracer->value;
		tracer_flags = axn_tracer_flags->value;
		
		if(tracer && IsAlive(pEntity) && i > 0
		&& g_PlayerInfo[index].curweap.id == id
		&& (Weapons[id] == 0 || Weapons[id] == 1)
		&& clip < g_PlayerInfo[index].curweap.clip
		&& (!(tracer_flags & FL_TRACER_HIDE_SNIPER && isSniperWeapon(id)))
		&& (!(tracer_flags & FL_TRACER_HIDE_SILENCED && isWeaponSilenced(index, id))))
		{
			static bool bRecalc;
			static TraceResult tr;
			static pfnSendTracer fnSendTracer;

			Vector v_angle = pEntity->v.v_angle;

			if(v_angle.y > 180)
				v_angle.y -= 360;
			if(v_angle.x > 180)
				v_angle.x -= 360;
			
			UTIL_MakeVectors(v_angle);
			
			Vector vecDir = gpGlobals->v_forward;
			Vector vecStart = pEntity->v.origin + pEntity->v.view_ofs;
			Vector vecEnd = vecStart + vecDir * 8192.0;

			UTIL_TraceLine(vecStart, vecEnd, dont_ignore_monsters, pEntity->v.pContainingEntity, &tr);
			
			vecEnd = tr.vecEndPos;
			
			if(isShotgun(id))
				fnSendTracer = SendTracerShortgun;
			else if(g_PlayerInfo[index].curweap.clip > 1 && isWeaponInBurstMode(index, id))
				fnSendTracer = SendTracerBurst;
			else
				fnSendTracer = SendTracer;
			
			bRecalc = true;

			if(tracer_flags & FL_TRACER_PLAYER)
			{
				if(tracer_flags & FL_TRACER_HIDE_OWN)
				{
					// send tracer to everybody except shooter...
					for(i = 1; i<=gpGlobals->maxClients; i++)
					{
						if(i == index) continue;
					
						pEntity = INDEXENT(i);
						if(pEntity && !pEntity->free && IsAlive(pEntity))
						{
							fnSendTracer(/*MSG_ONE*/MSG_ONE_UNRELIABLE, pEntity, vecStart, vecEnd, bRecalc);
						}
					}
				}
				else
					fnSendTracer(MSG_PAS, NULL, vecStart, vecEnd, bRecalc);
			}
			
			if(tracer_flags & FL_TRACER_HLTV)
			{
				fnSendTracer(MSG_SPEC, NULL, vecStart, vecEnd, bRecalc);
			}
		}
		
		// update client weapon data...
		g_PlayerInfo[index].curweap.id = id;
		g_PlayerInfo[index].curweap.clip = clip;
		
		break;
	}

	return false;
}
//-----------------------------------------------------------------------------
bool MsgFunc_Damage(void *value)
{
	static int index = 0;
	static edict_t *pEntity;
	static Vector origin;
	
	if(g_msg_index == MSGBEGIN)
	{
		pEntity=(edict_t *)value;
		index=ENTINDEX(pEntity);
	}
	else if(g_msg_index==4)
	{
		origin.x=*(float *)value;
	}
	else if(g_msg_index==5)
	{
		origin.y=*(float *)value;
	}
	else if(g_msg_index==6)
	{
		origin.z=*(float *)value;
	}
	else if(g_msg_index == MSGEND && axn_damageimpact->value > 0.0)
	{	// implementation of recoil on player hit...
		static Vector impact;
		
		impact = (pEntity->v.origin-origin).Normalize()*pEntity->v.dmg_take*axn_damageimpact->value;
		pEntity->v.velocity = pEntity->v.velocity+impact;
	}
	
	return false;
}
//-----------------------------------------------------------------------------
bool MsgFunc_ScreenFade(void *value)
{
	static int index = 0;
	static edict_t *pEntity;
	
	if(g_msg_index == MSGBEGIN)
	{
		pEntity=(edict_t *)value;
		index=ENTINDEX(pEntity);
	}
	else if(g_msg_index == 2 && axn_flashtime->value >= 0.0)
	{
		int holdTime=*(int *)value;
		*(int *)value = holdTime * axn_flashtime->value;
		
		return true;
	}
	
	return false;
}
//-----------------------------------------------------------------------------
bool MsgFunc_StatusText(void *value)
{
	static int index=0;
	static edict_t *pEntity;

	if(g_msg_index == MSGBEGIN)
	{
		pEntity = (edict_t *)value;
		index = ENTINDEX(pEntity);
	}
	else if(g_msg_index == 2)
	{
		if(IsAlive(pEntity) && axn_statustext->value > 0)
		{
			char *string = (char*)value;
			strcpy(string, "");
			return true;
		}
	}
	
	return false;
}
//-----------------------------------------------------------------------------
bool MsgFunc_WeaponList(void *value)
{
	static int 
		wid = 0,
		iProceeded = 0, // bitsum
		iSlot = 0;
	static const char *pszWeaponName;

	if(g_msg_index == 1) // weapon classname
	{
		pszWeaponName = (char*)value;
	}
	else if(g_msg_index == 6) // weapon slot
	{
		iSlot = *((int*)value);
	}
	else if(g_msg_index == 8) // weapon id
	{
		wid = *(int*)value;
		if((wid < 0 || wid >= MAX_WEAPONS) || (iProceeded & (1<<wid)))
			return false;
		
		Weapons[wid] = iSlot;
		strncpy(g_szWeaponNames[wid], pszWeaponName, 31);
		if(g_szWeaponNames[wid][31]) // for safety
			g_szWeaponNames[wid][31] = '\0';

		iProceeded |= (1<<wid);

		//LOG_CONSOLE(PLID, "New weapon parsed: %s", pszWeaponName);
	}

	return false;
}
//-----------------------------------------------------------------------------
/*
bool MsgFunc_TextMsg(void *value)
{
	// TODO: trigger that grenade has been thrown (Fire_in_the_hole)?
	// TODO: get grenade type and if this is a gas grenade - register it's id

	switch(g_msg_index)
	{
	
	}


	return false;
}
*/
//-----------------------------------------------------------------------------
